/**************************************************************************
*
* Filename:     CAAccounts.cs
* Author:       Andreas Kerl (andreaskerl@msn.com)
*               http://www.microsoft.com/germany/
*
* Description:  Verwalten der Benutzerkonten
*				
*
*
* Copyright 2005, Andreas Kerl
* 
**************************************************************************/

namespace ManagedMSI
{
	using System;
	using System.Text;
	using System.DirectoryServices;
	using System.Runtime.InteropServices;

	/// <summary>
	/// Klasse zur Verwaltung der Benuzers
	/// </summary>
	public class ManageAccounts
	{
		#region Konstanten

		// Flags fr die User-Properties
		const int UF_PASSWD_CANT_CHANGE        = 0x0040;
		const int UF_NORMAL_ACCOUNT            = 0x0200;

		#endregion

		#region Methoden
		/// <summary>
		/// Ermittelt die Benutzerkonten des lokalen Systems
		/// </summary>
		/// <returns>Gibt die Benutzers als ArrayList zurck</returns>
		public static System.Collections.ArrayList EnumerateAccounts()
		{
			// ArrayList definieren
			System.Collections.ArrayList userList = new System.Collections.ArrayList(); 

			// Variablendeklaration
			int EntriesRead;
			int TotalEntries;
			int Resume;

			// Buffer deklarieren und User enumerieren
			IntPtr bufPtr;
			NetUserEnum(null, 0, 2, out bufPtr, -1, out EntriesRead, out TotalEntries, out Resume);

			// Iteration durch alle Benutzer
			if(EntriesRead> 0)
			{
				USER_INFO_0[] Users = new USER_INFO_0[EntriesRead];
				IntPtr iter = bufPtr;
				for(int i=0; i < EntriesRead; i++)
				{
					Users[i] = (USER_INFO_0)Marshal.PtrToStructure(iter, typeof(USER_INFO_0)); 
					iter = (IntPtr)((int)iter + Marshal.SizeOf(typeof(USER_INFO_0)));

					// Benutzer der Liste anfgen
					userList.Add(new UserAccount(Users[i].Username, GetSIDFromUser(Users[i].Username))); 
				}
				NetApiBufferFree(bufPtr);
			}

			return (userList);
		}

		/// <summary>
		/// Account erstellen
		/// </summary>
		/// <param name="userName">AccountName</param>
		/// <param name="description">Beschreibung</param>
		/// <param name="password">Passwort</param>
		public static void CreateAccount(string userName, string description, string password)
		{
			// DirectoryEntry festlegen
			DirectoryEntry mainEntry = new DirectoryEntry("WinNT://" + Environment.MachineName);

			try    
			{
				// Neuen Account erstellen
				DirectoryEntry newUser = mainEntry.Children.Add(userName, "user");
				newUser.Properties["description"].Add(description);
				
				// Account-Properties: Normal User und Passwort kann nicht gendert werden
				newUser.Properties["userFlags"].Add(UF_NORMAL_ACCOUNT|UF_PASSWD_CANT_CHANGE);
				
				// Passwort setzen
				newUser.Invoke("SetPassword", new Object[] {password});
				newUser.CommitChanges();

			}
			catch(COMException ex)    
			{
				throw new ApplicationException(ex.Message);
			}
			finally
			{
				mainEntry.Close();
			}
		}


		/// <summary>
		/// Benutzeraccount entfernen
		/// </summary>
		/// <param name="userName">Name des Accounts</param>
		public static void RemoveAccount(string userName)
		{
			// DirectoryEntry festlegen
			DirectoryEntry mainEntry = new DirectoryEntry("WinNT://" + Environment.MachineName);

			try    
			{
				// Konto suchen und entfernen
				DirectoryEntry oldUser = mainEntry.Children.Find(userName, "User");
				mainEntry.Children.Remove(oldUser);
			}
			catch 			
			{
				// Exception
				throw new ApplicationException(string.Format("Der Account \"{0}\" existiert nicht.", userName));
			}
			finally
			{
				mainEntry.Close();
			}
		}

		/// <summary>
		/// Ermittelt die SID vom Namen des Benutzers
		/// </summary>
		/// <param name="AccountName">Name des Benutzers</param>
		/// <returns>Gibt die SID zurck</returns>
		private static string GetSIDFromUser(string AccountName)
		{
			IntPtr bufPtr;
			USER_INFO_4 CurrentUser = new USER_INFO_4();
			NetUserGetInfo(null, AccountName, 4, out bufPtr);
			CurrentUser = (USER_INFO_4)Marshal.PtrToStructure(bufPtr, typeof(USER_INFO_4)); 

			// Rckgabe
			return ( GetSIDString(CurrentUser.usri4_user_sid)); 
		}

		/// <summary>
		/// Wandelt den SID-Pointer in eine Zeichenfolge um
		/// </summary>
		/// <param name="pSid">Pointer auf die SID</param>
		/// <returns>Formatierte Zeichenfolge</returns>
		private static string GetSIDString(IntPtr pSid)
		{
			string szSID = string.Empty ;
			
			// Konvertieren
			ConvertSidToStringSid(pSid, ref szSID);

			// Rckgabe
			return (szSID);
		}
		#endregion
	
		#region Native Funktionen

		/// <summary>
		/// Win32-API zur Ermittlung des Accounts anhand der SID
		/// </summary>
		/// <param name="lpSystemName">Name des lokalen oder Remote Computers</param>
		/// <param name="pSid">SID</param>
		/// <param name="Account">Buffer fr den Account</param>
		/// <param name="cbName">Gre des Buffers fr den Account</param>
		/// <param name="DomainName">Buffer fr den Domnennamen</param>
		/// <param name="cbDomainName">Gre des Buffers fr den Domnennamen</param>
		/// <param name="peUse">SID-Typ</param>
		/// <returns>Gibt True zurck, falls alles erfolgreich verlaufen ist</returns>
		[DllImport("advapi32", CharSet=CharSet.Auto)]
		private static extern bool LookupAccountSid
			(
			[In,MarshalAs(UnmanagedType.LPTStr)] string lpSystemName,
			IntPtr pSid,
			StringBuilder Account,
			ref int cbName,
			StringBuilder  DomainName,
			ref int cbDomainName,
			ref int peUse
			);


		/// <summary>
		/// Ermitteltdie SID fr einen Account
		/// </summary>
		/// <param name="ServerName">Name des Servers</param>
		/// <param name="AccountName">Name des Accounts</param>
		/// <param name="Sid">Pointer auf die SID</param>
		/// <param name="SidSize">Gre des Buffers fr die SID</param>
		/// <param name="DomainName">Buffer fr den Domnennamen</param>
		/// <param name="DomainNameSize">Gre des Buffers fr den Domnennamen</param>
		/// <param name="SidUse">Filter fr die SIDs</param>
		/// <returns>Bei erfolgreicher Ausfhrung ein Wert != 0</returns>
		[DllImport("Advapi32.dll",  SetLastError=true)]
		private static extern int LookupAccountName(
			string ServerName,
			string AccountName,
			IntPtr Sid,
			ref int SidSize,
			StringBuilder DomainName,
			ref int DomainNameSize,
			ref short SidUse);

		/// <summary>
		/// Konvertiert eine SID in eine lesbare Zeichenfolge
		/// </summary>
		/// <param name="pSID">Pointer auf die SID</param>
		/// <param name="pStringSid">Ermittelte Zeichenfolge</param>
		/// <returns>Bei erfolgreicher Ausfhrung, True</returns>
		[DllImport("advapi32", CharSet=CharSet.Auto)]
		private static extern bool ConvertSidToStringSid(
			IntPtr pSID,
			[In,Out,MarshalAs(UnmanagedType.LPTStr)] ref string pStringSid);

		/// <summary>
		/// Konvertiert eine lesbare Zeichenfolge in eine SID
		/// </summary>
		/// <param name="pStringSid">Lesbare Zeichenfolge</param>
		/// <param name="pSID">Ermittelter Pointer auf die SID</param>
		/// <returns>Bei erfolgreicher Ausfhrung, True</returns>
		[DllImport("advapi32", CharSet=CharSet.Auto)]
		private static extern bool ConvertStringSidToSid(
			string pStringSid,
			ref IntPtr pSID);

		// NetUserEnum - Obtains a list of all users on local machine or network
		
		/// <summary>
		/// Ermittelt alle Benutzer der lokalen Maschine oder im Netzwerk
		/// </summary>
		/// <param name="servername">Name des Servers</param>
		/// <param name="level">Level</param>
		/// <param name="filter">Filter</param>
		/// <param name="bufptr">Buffer zur Aufnahme der Daten</param>
		/// <param name="prefmaxlen">Maximale Buffergre</param>
		/// <param name="entriesread">Anzahl der enumerierten Elemente</param>
		/// <param name="totalentries">Gesamtanzahl der Elemente</param>
		/// <param name="resume_handle">Handle</param>
		/// <returns>Bei erfolgreicher Ausfhrung ein Wert != 0</returns>
		[DllImport("Netapi32.dll")]
		private extern static int NetUserEnum(string servername, int level, int filter, out IntPtr bufptr, int prefmaxlen, out int entriesread, out int totalentries, out int resume_handle);

		/// <summary>
		/// Gibt den Buffer wieder frei 
		/// </summary>
		/// <param name="Buffer">Buffer der freigegeben werden soll</param>
		/// <returns>Bei erfolgreicher Ausfhrung ein Wert != 0</returns>
		[DllImport("Netapi32.dll")]
		private extern static int NetApiBufferFree(IntPtr Buffer);

		/// <summary>
		/// Gibt Informationen zu dem Benutzer zurck
		/// </summary>
		/// <param name="servername">Name des servers</param>
		/// <param name="username">Name des Benutzers</param>
		/// <param name="level">Level</param>
		/// <param name="bufptr">Buffer zur Aufnahme der Daten</param>
		/// <returns>Bei erfolgreicher Ausfhrung ein Wert != 0</returns>
		[DllImport("Netapi32.dll")]
		private extern static int NetUserGetInfo([MarshalAs(UnmanagedType.LPWStr)] string servername,[MarshalAs(UnmanagedType.LPWStr)] string username,int level,out IntPtr bufptr);

		#endregion

		#region Native Strukturen
		/// <summary>
		/// Struktur zur Aufnahme des Benutzernamens
		/// </summary>
		[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Unicode)]
			private struct USER_INFO_0
		{
			public String Username;
		}

		/// <summary>
		/// Struktur zur Aufnahme des Benutzernamens, sowie zustzlichen Daten
		/// </summary>
		[StructLayout(LayoutKind.Sequential)]
			private struct USER_INFO_4
		{
			[MarshalAs(UnmanagedType.LPWStr)] public string usri4_name;
			[MarshalAs(UnmanagedType.LPWStr)] public string usri4_password;
			[MarshalAs(UnmanagedType.U4)] public int usri4_password_age;
			[MarshalAs(UnmanagedType.U4)] public int usri4_priv;
			[MarshalAs(UnmanagedType.LPWStr)] public string usri4_home_dir;
			[MarshalAs(UnmanagedType.LPWStr)] public string usri4_comment;
			[MarshalAs(UnmanagedType.U4)] public int usri4_flags;
			[MarshalAs(UnmanagedType.LPWStr)] public string usri4_script_path;
			[MarshalAs(UnmanagedType.U4)] public int usri4_auth_flags;
			[MarshalAs(UnmanagedType.LPWStr)] public string usri4_full_name;
			[MarshalAs(UnmanagedType.LPWStr)] public string usri4_usr_comment;
			[MarshalAs(UnmanagedType.LPWStr)] public string usri4_parms;
			[MarshalAs(UnmanagedType.LPWStr)] public string usri4_workstations;
			[MarshalAs(UnmanagedType.U4)] public int usri4_last_logon;
			[MarshalAs(UnmanagedType.U4)] public int usri4_last_logoff;
			[MarshalAs(UnmanagedType.U4)] public int usri4_acct_expires;
			[MarshalAs(UnmanagedType.U4)] public int usri4_max_storage;
			[MarshalAs(UnmanagedType.U4)] public int usri4_units_per_week;
			public IntPtr usri4_logon_hours;
			[MarshalAs(UnmanagedType.U4)] public int usri4_bad_pw_count;
			[MarshalAs(UnmanagedType.U4)] public int usri4_num_logons;
			[MarshalAs(UnmanagedType.LPWStr)] public string usri4_logon_server;
			[MarshalAs(UnmanagedType.U4)] public int usri4_country_code;
			[MarshalAs(UnmanagedType.U4)] public int usri4_code_page;
			public IntPtr usri4_user_sid;
			[MarshalAs(UnmanagedType.U4)] public int usri4_primary_group_id;
			[MarshalAs(UnmanagedType.LPWStr)] public string usri4_profile;
			[MarshalAs(UnmanagedType.LPWStr)] public string usri4_home_dir_drive;
			[MarshalAs(UnmanagedType.U4)] public int usri4_password_expired;
		}
		#endregion
	}

	/// <summary>
	/// Klasse zur Definition der Benutzers
	/// </summary>
	public class UserAccount
	{
		#region Properties
		/// <summary>
		/// Gibt den Namen des Benutzers zurck
		/// </summary>
		public string Name
		{
			get	{return this.name;}
		}
		private string name;

		/// <summary>
		/// Gibt die SID des Benutzers zurck
		/// </summary>
		public string SID
		{
			get	{return this.sid;	}
		}
		private string sid;

		#endregion

		#region CTOR
		/// <summary>
		/// CTOR
		/// </summary>
		/// <param name="userName">Name des Benutzers</param>
		/// <param name="userSID">SID des Benutzers</param>
		public UserAccount(string userName, string userSID)
		{
			this.name = userName;
			this.sid = userSID;
		}
		#endregion

		#region Methoden
		/// <summary>
		/// ToString()
		/// </summary>
		/// <returns></returns>
		public override string ToString()
		{
			return string.Format("{0}; SID: {1}", this.name, this.sid);   
		}
		#endregion

	}

}
