/**************************************************************************
*
* Filename:     CAHelper.cs
* Author:       Andreas Kerl (andreaskerl@msn.com)
*               http://www.microsoft.com/germany/
*
* Description:  Funktionen zum Parsen der Befehlszeile
*				
*
*
* Copyright 2005, Andreas Kerl
* 
**************************************************************************/

namespace ManagedMSI
{
	using System;
	using System.Text; 
	using System.Globalization;
	using System.Collections;
	using System.Collections.Specialized;
	
	/// <summary>
	/// Funktionen zum Parsen der Befehlszeile
	/// </summary>
	public class Parser
	{
		#region Parser
		/// <summary>
		/// Befehlszeile parsen
		/// </summary>
		/// <param name="cmdLine">Befehlszeile</param>
		/// <returns>StringDictionary</returns>
		internal static StringDictionary ParseCommandLine(string cmdLine) 
		{
			string[] args = StringToArgs(cmdLine);
			return (ParseCommandLine(args));
		}

		/// <summary>
		/// Befehlszeilenargumente parsen
		/// </summary>
		/// <param name="args">Argumente</param>
		/// <returns>StringDictionary</returns>
		internal static StringDictionary ParseCommandLine(string[] args) 
		{
			StringDictionary options = new StringDictionary();
			if (args == null)
				return (options);
			
			for (int i = 0; i < args.Length; i++) 
			{
				// Entfernen von fhrenden / oder -
				if (args[i].StartsWith("/") || args[i].StartsWith("-"))
					args[i] = args[i].Substring(1);
				// Ermitteln von '='
				int equalsPos = args[i].IndexOf('=');
				// Bei keinen bereinstimmungen einen Parameter mit einem Wert von "" anfgen
				if (equalsPos < 0)
					options[args[i].ToLower(CultureInfo.InvariantCulture)] = string.Empty;
				else
					options[args[i].Substring(0, equalsPos).ToLower(CultureInfo.InvariantCulture)] = args[i].Substring(equalsPos+1);
			}
			
			return (options);
		}

		/// <summary>
		/// Zeichenfolge in Argumente umwandeln
		/// </summary>
		/// <param name="cmdLine">Zeichenfolge</param>
		/// <returns>String Array</returns>
		static string[] StringToArgs(string cmdLine) 
		{
			ArrayList args = new ArrayList();
			StringBuilder nextArg = null;
			bool inString = false;
			bool escaped = false;
            
			for (int pos = 0; pos < cmdLine.Length; pos++) 
			{
				char ch = cmdLine[pos];            
    
				if (nextArg == null) 
				{
					if (Char.IsWhiteSpace(ch)) 
						continue;
					else 
						nextArg = new StringBuilder();
				}
     
				if (inString) 
				{
					if (escaped) 
					{
						if (ch != '\\' && ch != '\"')                 
							nextArg.Append('\\');                     
						escaped = false;
						nextArg.Append(ch);  
					} 
					else if (ch == '\"') 
					{
						inString = false;
					} 
					else if (ch == '\\') 
					{
						escaped = true;
					} 
					else 
					{
						nextArg.Append(ch);
					}
				}
				else 
				{
					if (Char.IsWhiteSpace(ch)) 
					{
						args.Add(nextArg.ToString());
						nextArg = null;              
						escaped = false;      
					} 
					else if (escaped) 
					{
						nextArg.Append(ch);
						escaped = false;
					} 
					else if (ch == '^') 
					{
						escaped = true;
					} 
					else if (ch == '\"') 
					{
						inString = true;
					} 
					else 
					{
						nextArg.Append(ch);
					}                
				}
                
			}
    
			if (nextArg != null)
				args.Add(nextArg.ToString());
    
			string[] argsArray = new string[args.Count];
			args.CopyTo(argsArray);
    
			return (argsArray);
		}
		#endregion
	}
}
