/**************************************************************************
*
* Filename:     CAClass.cs
* Author:       Andreas Kerl (andreaskerl@msn.com)
*               http://www.microsoft.com/germany/
*
* Description:  Implementierung zum Zugriff auf die Windows Installer Funktionen
*				
*
*
* Copyright 2005, Andreas Kerl
* 
**************************************************************************/

namespace PatchModell
{
	using System;
	using System.IO;
	using System.Windows.Forms; 

	/// <summary>
	/// Implementierung zum Zugriff auf die Windows Installer Funktionen
	/// </summary>
	public class CAClass
	{
		#region Konstanten
		/// <summary>
		/// Aktion wurde erfolgreich beendet
		/// </summary>
		private const int ERROR_SUCCESS = 0;

		/// <summary>
		/// User hat die Installation abgebrochen
		/// </summary>
		private const int ERROR_INSTALL_USEREXIT = 1602;

		/// <summary>
		/// Ein Fehler ist bei der Installation aufgetreten
		/// </summary>
		private const int ERROR_INSTALL_FAILURE = 1603;

		#endregion

		#region Methoden
		/// <summary>
		/// Ermittelt die Patches im Originalinstallationsverzeichnis und bergibt diese
		/// der Eigenschaft PATCH
		/// </summary>
		/// <param name="hSession">Windows Installer Session</param>
		/// <returns>Rckgabewert der Custom Action</returns>
		[DllExportAttribute()]
		public static int SetPatches(uint hSession)
		{
			try
			{
				// Produktcode und Installationsquelle bestimmen
				string productCode = MsiFunctions.GetProperty(hSession, "ProductCode");
				string sourceLocation = MsiFunctions.GetProductInfo(productCode, "InstallSource"); 

				// Prfen ob eine administrative Installation ausgefhrt wird
				bool isAdminPackage = (MsiFunctions.GetProperty(hSession, "IsAdminPackage") == "1");
				if (!isAdminPackage)
					throw new ApplicationException("Aktualisierung erfolgt nicht vom administrativen Abbild"); 

				// Installationsquelle prfen
				DirectoryInfo directoryInfo = new DirectoryInfo(sourceLocation);
				if (!directoryInfo.Exists)
					throw new DirectoryNotFoundException("Auf die Installationsquelle kann nicht zugegriffen werden"); 

				// Patches des Verzeichnisse bestimmen
				FileInfo[] fileInfos = directoryInfo.GetFiles("*.msp"); 
				if (fileInfos.Length == 0)
					throw new ApplicationException("Es wurden keine Patches gefunden");
 
				// Konstruieren des Eigenschaftswertes PATCH
				string patches = string.Empty; 
				foreach (FileInfo fileInfo in fileInfos)
					patches += fileInfo.FullName + ";";

				// Eigenschaftswert zuweisen
				MsiFunctions.SetProperty(hSession, "PATCH", patches); 

				// Rckgabe
				return (ERROR_SUCCESS);
			}
			catch (Exception ex)
			{
				// Info anzeigen
				MessageBox.Show(ex.Message, "Patches anwenden", MessageBoxButtons.OK, MessageBoxIcon.Error ); 

				// Rckgabe Fehler
				return (ERROR_INSTALL_FAILURE);
			}
		}
		#endregion
	}
}
