'**************************************************************************
'*
'* Filename:     CANative.vb
'* Author:       Andreas Kerl (andreaskerl@msn.com)
'*               http://www.microsoft.com/germany/
'*
'* Description:  Native Functions to access Windows Installer 
'*				
'*
'*
'* Copyright 2005, Andreas Kerl
'* 
'**************************************************************************/

Imports System.Text
Imports System.Runtime.InteropServices

#Region "Windows Installer Session"

''' <summary>
''' Functions to access Windows Installer properties
''' </summary>
Friend Class MsiFunctions

    ''' <summary>
    ''' Read Property
    ''' </summary>
    ''' <param name="Session">Handle to Windows Installer session</param>
    ''' <param name="Name">Name</param>
    ''' <returns>Value</returns>
    Friend Shared Function GetProperty(ByVal Session As Integer, ByVal Name As String) As String
        Dim hInstall As IntPtr = New IntPtr(Session)

        Dim bufSize As Integer = 255
        Dim buf As New System.Text.StringBuilder(bufSize)

        Dim ret As Integer = NativeMethods.MsiGetProperty(hInstall, Name, buf, bufSize)
        If ret <> 0 Then
            Throw New ApplicationException("Exception: " + ret)
        End If
        Return buf.ToString()
    End Function


    ''' <summary>
    ''' Save Property
    ''' </summary>
    ''' <param name="Session">Handle to Windows Installer session</param>
    ''' <param name="Name">Name</param>
    ''' <param name="Value">Value</param>
    Friend Shared Sub SetProperty(ByVal Session As Integer, ByVal Name As String, ByVal Value As String)
        Dim hInstall As IntPtr = New IntPtr(Session)

        Dim ret As Integer = NativeMethods.MsiSetProperty(hInstall, Name, Value)
        If ret <> 0 Then
            Throw New ApplicationException("Exception: " + ret)
        End If
    End Sub
End Class

#End Region

#Region "Native"

''' <summary>
''' Native functions
''' </summary>
Friend Class NativeMethods

    ''' <summary>
    ''' Read Property
    ''' </summary>
    Friend Declare Auto Function MsiGetProperty Lib "msi.dll" (ByVal hInstall As IntPtr, ByVal szName As String, ByVal szValueBuf As System.Text.StringBuilder, ByRef cchValueBuf As Integer) As Integer

    ''' <summary>
    ''' Save Property
    ''' </summary>
    Friend Declare Auto Function MsiSetProperty Lib "msi.dll" (ByVal hInstall As IntPtr, ByVal szName As String, ByVal szValue As String) As Integer

    ''' <summary>
    ''' Get ForeGroundWindow
    ''' </summary>
    ''' <returns>Handle</returns>
    Friend Declare Auto Function GetForegroundWindow Lib "user32.dll" () As Integer


    ''' <summary>
    ''' Errors
    ''' </summary>
    Friend Enum Errors As Integer
        SUCCESS = 0
        MORE_DATA = 234
    End Enum
End Class

#End Region

