/**************************************************************************
*
* Filename:     CANative.cs
* Author:       Andreas Kerl (andreaskerl@msn.com)
*               http://www.microsoft.com/germany/
*
* Description:  Native Functions to access Windows Installer 
*				
*
*
* Copyright 2005, Andreas Kerl
* 
**************************************************************************/

namespace CALibrary5
{
	using System;
	using System.Text;
	using System.Runtime.InteropServices;

	#region Windows Installer Session
	/// <summary>
	/// Functions to access Windows Installer properties
	/// </summary>
	internal class MsiFunctions
	{
		/// <summary>
		/// Read Property
		/// </summary>
		/// <param name="Session">Handle to Windows Installer session</param>
		/// <param name="Name">Name</param>
		/// <returns>Value</returns>
		internal static string GetProperty(uint Session, string Name)
		{	
			IntPtr hInstall = new IntPtr(Session);

			StringBuilder buf = new StringBuilder();
			uint bufSize = 0;
			uint ret = NativeMethods.MsiGetProperty(hInstall, Name, buf, ref bufSize);
			if(ret == (uint) NativeMethods.Error.MORE_DATA)
			{
				buf.Capacity = (int) ++bufSize;
				ret = NativeMethods.MsiGetProperty(hInstall, Name, buf, ref bufSize);
			}
			if(ret != 0)
			{
				throw new ApplicationException("Exception: " + ret); 
			}
			return buf.ToString();
		}

		/// <summary>
		/// Save Property
		/// </summary>
		/// <param name="Session">Handle to Windows Installer session</param>
		/// <param name="Name">Name</param>
		/// <param name="Value">Value</param>
		internal static void SetProperty(uint Session, string Name, string Value)
		{
			IntPtr hInstall = new IntPtr(Session);

			uint ret = NativeMethods.MsiSetProperty(hInstall, Name, Value);
			if(ret != 0)
			{
				throw new ApplicationException("Exception: " + ret); 
			}
		}
	}
	#endregion

	#region Native
	/// <summary>
	/// Native functions
	/// </summary>
	internal sealed class NativeMethods
	{
		/// <summary>
		/// Read Property
		/// </summary>
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] internal static extern uint MsiGetProperty(IntPtr hInstall, string szName, StringBuilder szValueBuf, ref uint cchValueBuf);
		
		/// <summary>
		/// Save Property
		/// </summary>
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] internal static extern uint MsiSetProperty(IntPtr hInstall, string szName, string szValue);

		/// <summary>
		/// Get ForeGroundWindow
		/// </summary>
		/// <returns>Handle</returns>
		[DllImport("user32.dll")] internal  static extern int GetForegroundWindow();


		/// <summary>
		/// Errors
		/// </summary>
		internal enum Error : uint
		{
			SUCCESS                =    0,
			MORE_DATA              =  234,

		}
	}
	#endregion
}
