/**************************************************************************
*
* Filename:     NativeMsi.cs
* Author:       Andreas Kerl (andreaskerl@msn.com)
*               http://www.microsoft.com/germany/
*
* Description:  Enthlt native Funktionen fr den Windows Installer
*				
*
*
* Copyright 2005, Andreas Kerl
* 
**************************************************************************/

namespace Darwin.Common
{
	using System;
	using System.Text; 
	using System.Runtime.InteropServices;

	/// <summary>
	/// Enthlt native Funktionen fr den Windows Installer
	/// </summary>
	internal class NativeMsi
	{
		#region Methoden
		/// <summary>
		/// Ermittelt den Pfad zu einer Komponente
		/// </summary>
		/// <param name="product">ProductCode</param>
		/// <param name="feature">Feature</param>
		/// <param name="component">ComponentId</param>
		/// <returns>Gibt den Pfad zurck</returns>
		internal static string ProvideComponent(string product, string feature, string component)
		{
			// InstallMode = Default 
			uint installMode = 0;

			StringBuilder pathBuf = new StringBuilder(512);
			uint pathBufSize = (uint) pathBuf.Capacity;
			uint ret = MsiProvideComponent(product, feature, component, (uint) installMode, pathBuf, ref pathBufSize);
			if(ret == (uint)234)
			{
				pathBuf.Capacity = (int) ++pathBufSize;
				ret = MsiProvideComponent(product, feature, component, (uint) installMode, pathBuf, ref pathBufSize);
			}
			if(ret != 0)
			{
				throw new ApplicationException(GetSystemMessage(ret)); 
			}
			return pathBuf.ToString();
		}



		/// <summary>
		/// Ermittelt eine Systemfehlermeldung
		/// </summary>
		/// <param name="errorCode">Code fr den die Meldung generiert werden soll</param>
		/// <returns>Gibt die Fehlermeldung zurck</returns>
		internal static string GetSystemMessage(uint errorCode)
		{
			const uint FORMAT_MESSAGE_IGNORE_INSERTS  = 0x00000200;
			const uint FORMAT_MESSAGE_FROM_SYSTEM     = 0x00001000;
			StringBuilder buf = new StringBuilder(1024);
			if(FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
				IntPtr.Zero, (uint) errorCode, 0, buf, (uint) buf.Capacity, IntPtr.Zero) != 0)
			{
				return buf.ToString().Trim();
			}
			else
			{
				return null;
			}
		}

		#endregion

		#region Native Funktionen
		[DllImport("kernel32.dll", SetLastError=true)] 
		internal static extern uint FormatMessage(uint dwFlags, IntPtr lpSource, uint dwMessageId, uint dwLanguageId, StringBuilder lpBuffer, uint nSize, IntPtr Arguments);

		[DllImport("msi.dll", CharSet=CharSet.Unicode)] 
		internal static extern uint MsiProvideComponent(string szProduct, string szFeature, string szComponent, uint dwInstallMode, StringBuilder lpPathBuf, ref uint cchPathBuf);

		#endregion
	}
}
