/**************************************************************************
*
* Filename:     MsiFunctions.cs
* Author:       Andreas Kerl (andreaskerl@msn.com)
*               http://www.microsoft.com/germany/
*
* Description:  Zugriff auf Windows Installer-Funktionen
*				
*
*
* Copyright 2005, Andreas Kerl
* 
**************************************************************************/

namespace Advertise
{
	using System;
	using System.Text;
	using System.Runtime.InteropServices;

	#region Windows Installer Funktionen
	/// <summary>
	/// Klasse mit Windows Installer Funktionen
	/// </summary>
	public class MsiFunctions
	{
		#region Konstanten
		/// <summary>
		/// Funktion erfolgreich ausgefhrt
		/// </summary>
		private const uint ERROR_SUCCESS = 0;
		
		#endregion

		#region Native Funktionen
		/// <summary>
		/// Produkt ankndigen oder Skript erstellen
		/// </summary>
		[DllImport("msi.dll", CharSet=CharSet.Unicode)]
		internal static extern uint MsiAdvertiseProductEx(string szPackagePath, string szScriptFilePath, string szTransforms, ushort lgidLanguage, uint dwPlatform, uint dwOptions);

		/// <summary>
		/// Version des Windows Installers abrufen
		/// </summary>
		/// <param name="dvi"></param>
		[DllImport("msi.dll", CharSet=CharSet.Unicode)]
		internal static extern void DllGetVersion(uint[] dvi);

		#endregion

		#region Methoden


		/// <summary>
		/// Erstellt ein Skript fr die Produktankndigung
		/// </summary>
		/// <param name="packagePath">Pfad zum Windows Installer-Paket</param>
		/// <param name="scriptFilePath">Pfad zur Skriptdatei</param>
		/// <param name="transforms">Transformationen (Durch ; getrennt)</param>
		/// <param name="locale">LanguageID</param>
		/// <param name="platform">Plattform</param>
		/// <param name="instance">1 = Skript fr neue Produktinstanz, ansnsten 0</param>
		public static void GenerateAdvertiseScript(string packagePath, string scriptFilePath, string transforms, ushort locale, uint platform, uint instance)
		{
			// Funktion zum Erstellen des Skriptes aufrufen
			uint ret = MsiAdvertiseProductEx(packagePath, scriptFilePath, transforms, locale, platform, instance);
			if(ret != ERROR_SUCCESS)
			{
				// Exception
				throw new InstallerException(ret);
			}
		}

		/// <summary>
		/// Version des Windows Installer ermitteln
		/// </summary>
		/// <returns>ermittelte Version</returns>
		public static Version GetVersion()
		{
			// VersionInfostruktur initialisieren
			uint[] dllVersionInfo = new uint[5];
			dllVersionInfo[0] = 20;
			
			// Version ermitteln
			DllGetVersion(dllVersionInfo);
			
			// Rckgabe
			return (new Version((int) dllVersionInfo[1], (int) dllVersionInfo[2], (int) dllVersionInfo[3]));
		}

		#endregion
	}

	#endregion

	#region Exceptions

	/// <summary>
	/// Klasse zur Behandlung von nativen Exceptions
	/// </summary>
	[Serializable]
	public class InstallerException : SystemException
	{
		/// <summary>
		/// Message formatieren
		/// </summary>
		[DllImport("kernel32.dll", SetLastError=true)] 
		internal static extern uint FormatMessage(uint dwFlags, IntPtr lpSource, uint dwMessageId, uint dwLanguageId, StringBuilder lpBuffer, uint nSize, IntPtr Arguments);

		/// <summary>
		/// Instanz der Exception erstellen
		/// </summary>
		/// <param name="errorCode">Fehlercode</param>
		public InstallerException(uint errorCode):base(GetSystemMessage(errorCode)){}

		/// <summary>
		/// Systemfehlermeldung erzeugen
		/// </summary>
		/// <param name="errorCode">Fehlercode</param>
		/// <returns>Meldung</returns>
		private static string GetSystemMessage(uint errorCode)
		{
			const uint FORMAT_MESSAGE_IGNORE_INSERTS  = 0x00000200;
			const uint FORMAT_MESSAGE_FROM_SYSTEM     = 0x00001000;
			StringBuilder buf = new StringBuilder(1024);
			if(FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
				IntPtr.Zero, (uint) errorCode, 0, buf, (uint) buf.Capacity, IntPtr.Zero) != 0)
			{
				return buf.ToString().Trim();
			}
			else
			{
				return null;
			}
		}
	}

	#endregion
}
