/**************************************************************************
*
* Filename:     Native.cs
* Author:       Andreas Kerl (andreaskerl@msn.com)
*               http://www.microsoft.com/germany/
*
* Description:  Kapselung der nativen Funktionsaufrufe
*				
*
*
* Copyright 2005, Andreas Kerl
* 
**************************************************************************/

namespace CheckMutex
{
	using System;
	using System.Runtime.InteropServices; 

	/// <summary>
	/// Kapselung der nativen Funktionsaufrufe
	/// </summary>
	public class Native
	{
		#region Konstanten
		private const int STANDARD_RIGHTS_REQUIRED = 0x000F0000;
		private const int SYNCHRONIZE = 0x00100000;
		private const int MUTANT_QUERY_STATE = 0x0001;
		private const int MUTEX_ALL_ACCESS = (STANDARD_RIGHTS_REQUIRED | SYNCHRONIZE | MUTANT_QUERY_STATE);

		#endregion

		#region Native Funktionen
		/// <summary>
		/// Version des Windows Installers abrufen
		/// </summary>
		[DllImport("msi.dll", CharSet=CharSet.Unicode)]
		private static extern void DllGetVersion(uint[] dvi);

		/// <summary>
		/// Mutex ffnen
		/// </summary>
		[DllImport("kernel32.dll", SetLastError=true)] 
		private static extern int OpenMutexA ( int dwDesiredAccess, int bInheritHandle, string lpName); 

		/// <summary>
		/// Handle schlieen
		/// </summary>
		[DllImport("kernel32.dll", SetLastError=true)] 
		private static extern int CloseHandle ( int hObject);

		#endregion

		#region Methoden
		/// <summary>
		/// berprfen der Existenz des Mutex
		/// </summary>
		/// <returns>True, falls Mutex existiert</returns>
		public static bool CheckExecuteMutex()
		{
			int handle = OpenMutexA(MUTEX_ALL_ACCESS, 0, "_MSIExecute");

			if (handle == 0)
				return false;

			CloseHandle(handle);

			return true;
		}


		/// <summary>
		/// Version des Windows Installer ermitteln
		/// </summary>
		/// <returns>ermittelte Version</returns>
		public static Version GetVersion()
		{
			// VersionInfostruktur initialisieren
			uint[] dllVersionInfo = new uint[5];
			dllVersionInfo[0] = 20;
			
			// Version ermitteln
			DllGetVersion(dllVersionInfo);
			
			// Rckgabe
			return (new Version((int) dllVersionInfo[1], (int) dllVersionInfo[2], (int) dllVersionInfo[3]));
		}
		#endregion
	}
}
