/**************************************************************************
*
* Filename:     MsiForm.cs
* Author:       Andreas Kerl (andreaskerl@msn.com)
*               http://www.microsoft.com/germany/
*
* Description:  Formular zum Starten der Installation
*				
*
*
* Copyright 2005, Andreas Kerl
* 
**************************************************************************/

namespace Installation
{
	using System;
	using System.IO;
	using System.Windows.Forms;

	/// <summary>
	/// Formular zum Starten der Installation
	/// </summary>
	public class MsiForm : System.Windows.Forms.Form
	{
		#region Controls

		private System.Windows.Forms.TextBox txtFile;
		private System.Windows.Forms.Button btnBrowse;
		private System.Windows.Forms.Button btnStart;
		private System.Windows.Forms.Button btnCancel;
		private System.Windows.Forms.TextBox txtLog;
		private System.Windows.Forms.GroupBox gpMain;
		private System.Windows.Forms.StatusBar stBar;
		private System.Windows.Forms.StatusBarPanel panelVersion;
		private System.Windows.Forms.ComboBox cmbTyp;
		private System.Windows.Forms.Label lblFile;
		private System.Windows.Forms.Label lblType;
		private System.Windows.Forms.Label lblLog;
		private System.ComponentModel.Container components = null;

		#endregion

		#region Initialisierung / Finalisierung
		
		/// <summary>
		/// Erstellt eine Instanz des Objektes
		/// </summary>
		/// <param name="fileName">Name der Datei</param>
		public MsiForm(string fileName)
		{
			// Komponenten initialisierem
			InitializeComponent();

			// Dateiname bergeben
			this.txtFile.Text = fileName; 

			// Installationstyp hinzufpegn
			this.cmbTyp.Items.AddRange(new string[] {"Standardinstallation", "Administrative Installation", "Advertised Installation"});
			this.cmbTyp.SelectedIndex = 0;


			// Version des MSI ermitteln
			this.panelVersion.Text = "Windows Installer-Version: " + MsiFunctions.GetVersion().ToString();



		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}
		#endregion

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(MsiForm));
			this.txtFile = new System.Windows.Forms.TextBox();
			this.btnBrowse = new System.Windows.Forms.Button();
			this.btnStart = new System.Windows.Forms.Button();
			this.btnCancel = new System.Windows.Forms.Button();
			this.txtLog = new System.Windows.Forms.TextBox();
			this.gpMain = new System.Windows.Forms.GroupBox();
			this.lblLog = new System.Windows.Forms.Label();
			this.lblType = new System.Windows.Forms.Label();
			this.lblFile = new System.Windows.Forms.Label();
			this.cmbTyp = new System.Windows.Forms.ComboBox();
			this.stBar = new System.Windows.Forms.StatusBar();
			this.panelVersion = new System.Windows.Forms.StatusBarPanel();
			this.gpMain.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.panelVersion)).BeginInit();
			this.SuspendLayout();
			// 
			// txtFile
			// 
			this.txtFile.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.txtFile.Location = new System.Drawing.Point(104, 33);
			this.txtFile.Name = "txtFile";
			this.txtFile.Size = new System.Drawing.Size(360, 21);
			this.txtFile.TabIndex = 1;
			this.txtFile.Text = "";
			this.txtFile.TextChanged += new System.EventHandler(this.txtFile_TextChanged);
			// 
			// btnBrowse
			// 
			this.btnBrowse.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.btnBrowse.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnBrowse.Location = new System.Drawing.Point(472, 33);
			this.btnBrowse.Name = "btnBrowse";
			this.btnBrowse.Size = new System.Drawing.Size(32, 23);
			this.btnBrowse.TabIndex = 2;
			this.btnBrowse.Text = "...";
			this.btnBrowse.Click += new System.EventHandler(this.btnBrowse_Click);
			// 
			// btnStart
			// 
			this.btnStart.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.btnStart.Enabled = false;
			this.btnStart.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnStart.Location = new System.Drawing.Point(544, 16);
			this.btnStart.Name = "btnStart";
			this.btnStart.TabIndex = 1;
			this.btnStart.Text = "&Starten...";
			this.btnStart.Click += new System.EventHandler(this.btnStart_Click);
			// 
			// btnCancel
			// 
			this.btnCancel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.btnCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.btnCancel.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnCancel.Location = new System.Drawing.Point(544, 48);
			this.btnCancel.Name = "btnCancel";
			this.btnCancel.TabIndex = 2;
			this.btnCancel.Text = "B&eenden";
			this.btnCancel.Click += new System.EventHandler(this.btnCancel_Click);
			// 
			// txtLog
			// 
			this.txtLog.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.txtLog.BackColor = System.Drawing.SystemColors.Window;
			this.txtLog.Location = new System.Drawing.Point(104, 97);
			this.txtLog.Multiline = true;
			this.txtLog.Name = "txtLog";
			this.txtLog.ReadOnly = true;
			this.txtLog.ScrollBars = System.Windows.Forms.ScrollBars.Both;
			this.txtLog.Size = new System.Drawing.Size(400, 215);
			this.txtLog.TabIndex = 6;
			this.txtLog.Text = "";
			this.txtLog.WordWrap = false;
			// 
			// gpMain
			// 
			this.gpMain.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.gpMain.Controls.Add(this.lblLog);
			this.gpMain.Controls.Add(this.lblType);
			this.gpMain.Controls.Add(this.lblFile);
			this.gpMain.Controls.Add(this.cmbTyp);
			this.gpMain.Controls.Add(this.txtFile);
			this.gpMain.Controls.Add(this.txtLog);
			this.gpMain.Controls.Add(this.btnBrowse);
			this.gpMain.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.gpMain.Location = new System.Drawing.Point(16, 8);
			this.gpMain.Name = "gpMain";
			this.gpMain.Size = new System.Drawing.Size(520, 328);
			this.gpMain.TabIndex = 0;
			this.gpMain.TabStop = false;
			this.gpMain.Text = "Informationen zum Installationsprozess";
			// 
			// lblLog
			// 
			this.lblLog.AutoSize = true;
			this.lblLog.Location = new System.Drawing.Point(8, 96);
			this.lblLog.Name = "lblLog";
			this.lblLog.Size = new System.Drawing.Size(52, 17);
			this.lblLog.TabIndex = 5;
			this.lblLog.Text = "&Protokoll:";
			// 
			// lblType
			// 
			this.lblType.AutoSize = true;
			this.lblType.Location = new System.Drawing.Point(8, 66);
			this.lblType.Name = "lblType";
			this.lblType.Size = new System.Drawing.Size(82, 17);
			this.lblType.TabIndex = 3;
			this.lblType.Text = "Installations&art:";
			// 
			// lblFile
			// 
			this.lblFile.AutoSize = true;
			this.lblFile.Location = new System.Drawing.Point(8, 34);
			this.lblFile.Name = "lblFile";
			this.lblFile.Size = new System.Drawing.Size(96, 17);
			this.lblFile.TabIndex = 0;
			this.lblFile.Text = "&Installationspaket:";
			// 
			// cmbTyp
			// 
			this.cmbTyp.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.cmbTyp.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.cmbTyp.Location = new System.Drawing.Point(104, 65);
			this.cmbTyp.Name = "cmbTyp";
			this.cmbTyp.Size = new System.Drawing.Size(360, 21);
			this.cmbTyp.TabIndex = 4;
			// 
			// stBar
			// 
			this.stBar.Location = new System.Drawing.Point(0, 349);
			this.stBar.Name = "stBar";
			this.stBar.Panels.AddRange(new System.Windows.Forms.StatusBarPanel[] {
																					 this.panelVersion});
			this.stBar.ShowPanels = true;
			this.stBar.Size = new System.Drawing.Size(632, 22);
			this.stBar.TabIndex = 3;
			// 
			// panelVersion
			// 
			this.panelVersion.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Spring;
			this.panelVersion.BorderStyle = System.Windows.Forms.StatusBarPanelBorderStyle.None;
			this.panelVersion.Icon = ((System.Drawing.Icon)(resources.GetObject("panelVersion.Icon")));
			this.panelVersion.Width = 616;
			// 
			// MsiForm
			// 
			this.AcceptButton = this.btnStart;
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 14);
			this.CancelButton = this.btnCancel;
			this.ClientSize = new System.Drawing.Size(632, 371);
			this.Controls.Add(this.stBar);
			this.Controls.Add(this.gpMain);
			this.Controls.Add(this.btnCancel);
			this.Controls.Add(this.btnStart);
			this.Font = new System.Drawing.Font("Tahoma", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.MinimumSize = new System.Drawing.Size(368, 180);
			this.Name = "MsiForm";
			this.Text = "Windows Installer 3.1: Installation starten";
			this.gpMain.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.panelVersion)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

		#region Main
		/// <summary>
		/// Haupteinstiegspunkt der Anwendung
		/// </summary>
		[STAThread]
		static void Main(string[] args) 
		{
			string fileName = string.Empty; 

			// Argumente auswerten
			if (args.Length > 0) 
				fileName = args[0];


			// Styles aktivieren und Form anzeigen
			Application.EnableVisualStyles();
			Application.DoEvents(); 
			Application.Run(new MsiForm(fileName));
		}
		#endregion

		#region Methoden
		/// <summary>
		/// Auswhlen einer Datei wurde aktiviert
		/// </summary>
		private void btnBrowse_Click(object sender, System.EventArgs e)
		{
			using (OpenFileDialog dlgOpen = new OpenFileDialog())
			{
				// Dialog initialisieren
				dlgOpen.Title = "Windows Installer-Datei auswhlen";
				dlgOpen.Filter = "Windows Installer-Datei (*.msi)|*.msi";
				dlgOpen.FileName = this.txtFile.Text;

				// Dialog anzeigen
				if (dlgOpen.ShowDialog() == DialogResult.OK)
				{
					// Datei in Textfeld anzeigen
					this.txtFile.Text = dlgOpen.FileName; 
				}
			}
 
		}

		/// <summary>
		/// Datei wurde gendert
		/// </summary>
		private void txtFile_TextChanged(object sender, System.EventArgs e)
		{
			// Schaltflche aktivieren / deaktivieren
			this.btnStart.Enabled = (this.txtFile.Text.Trim() != string.Empty);   
		}

		/// <summary>
		/// Abbrechen wurde aktiviert
		/// </summary>
		private void btnCancel_Click(object sender, System.EventArgs e)
		{
			
			// Form schlieen
			this.Close(); 
		}

		/// <summary>
		/// Installation starten
		/// </summary>
		private void btnStart_Click(object sender, System.EventArgs e)
		{
			string logFile = string.Empty; 

			try
			{
				// Dateiname festlegen
				string fileName = this.txtFile.Text; 

				if (!File.Exists(fileName))
				{
					// Datei nicht vorhanden
					throw new FileNotFoundException(string.Format("Die Datei \"{0}\" konnte nicht gefunden werden",fileName), fileName); 
				}

				// Logfile erstellen
				logFile = Path.ChangeExtension(fileName, ".log");

				// Befehlszeile aufbauen
				string commandLine = string.Empty; 

				if (this.cmbTyp.SelectedIndex == 1)
					commandLine = "ACTION=ADMIN";
				else if (this.cmbTyp.SelectedIndex == 2)
					commandLine = "ACTION=ADVERTISE";


				// Installation starten
				MsiFunctions.InstallProduct(fileName, commandLine, logFile, this.Handle,this.cmbTyp.SelectedIndex == 2 );  


			}
			catch (Exception ex)
			{
				// Exception
				MessageBox.Show(ex.Message, this.Text, MessageBoxButtons.OK, MessageBoxIcon.Information);     
			}
			finally
			{
				// Prfen ob Log existiert
				if (File.Exists( logFile ))
				{
					// Streamreader erstellen
					using (StreamReader reader = new StreamReader(logFile))
					{
						// Log in Textbox anzeigen
						this.txtLog.Text = reader.ReadToEnd();
					}
				}
			}
		}

		#endregion
	}
}
