/**************************************************************************
*
* Filename:     MsiForm.cs
* Author:       Andreas Kerl (andreaskerl@msn.com)
*               http://www.microsoft.com/germany/
*
* Description:  Formular zum Starten der Installation
*				
*
*
* Copyright 2005, Andreas Kerl
* 
**************************************************************************/

namespace Watcher
{
	using System;
	using System.IO;
	using System.Windows.Forms;

	/// <summary>
	/// Formular zum Starten der Installation
	/// </summary>
	public class MsiForm : System.Windows.Forms.Form
	{
		#region Controls
		private System.Windows.Forms.Label lblPath;
		private System.Windows.Forms.TextBox txtFile;
		private System.Windows.Forms.Button btnBrowse;
		private System.Windows.Forms.Button btnStart;
		private System.Windows.Forms.Button btnCancel;
		private System.Windows.Forms.GroupBox gpMain;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.StatusBar stBar;
		private System.Windows.Forms.StatusBarPanel panelVersion;
		private System.Windows.Forms.ComboBox cmbDrives;
		private System.Windows.Forms.TextBox txtBackupFolder;
		private System.Windows.Forms.CheckBox chkRollback;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.Label label3;
		private System.Windows.Forms.Button btnFolder;
		private System.IO.FileSystemWatcher fswScript;
		private System.IO.FileSystemWatcher fswRollback;
		private System.Windows.Forms.ListBox lstState;
		private System.Windows.Forms.RadioButton rdDefault;
		private System.Windows.Forms.RadioButton rdAdmin;
		private System.Windows.Forms.RadioButton rdAdvt;
		private System.ComponentModel.Container components = null;

		#endregion

		#region Initialisierung / Finalisierung
		
		/// <summary>
		/// Erstellt eine Instanz des Objektes
		/// </summary>
		/// <param name="fileName">Name der Datei</param>
		public MsiForm(string fileName)
		{
			// Komponenten initialisierem
			InitializeComponent();

			// Dateiname bergeben
			this.txtFile.Text = fileName; 

			// Laufwerke der Combobox hinzufgen
			this.cmbDrives.Items.AddRange(Environment.GetLogicalDrives()); 
			this.cmbDrives.SelectedIndex = 0;

			// Version des MSI ermitteln
			this.panelVersion.Text = "Windows Installer-Version: " + MsiFunctions.GetVersion().ToString();

		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}
		#endregion

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(MsiForm));
			this.txtFile = new System.Windows.Forms.TextBox();
			this.btnBrowse = new System.Windows.Forms.Button();
			this.lblPath = new System.Windows.Forms.Label();
			this.btnStart = new System.Windows.Forms.Button();
			this.btnCancel = new System.Windows.Forms.Button();
			this.gpMain = new System.Windows.Forms.GroupBox();
			this.rdAdvt = new System.Windows.Forms.RadioButton();
			this.rdAdmin = new System.Windows.Forms.RadioButton();
			this.rdDefault = new System.Windows.Forms.RadioButton();
			this.lstState = new System.Windows.Forms.ListBox();
			this.txtBackupFolder = new System.Windows.Forms.TextBox();
			this.btnFolder = new System.Windows.Forms.Button();
			this.label3 = new System.Windows.Forms.Label();
			this.label2 = new System.Windows.Forms.Label();
			this.chkRollback = new System.Windows.Forms.CheckBox();
			this.cmbDrives = new System.Windows.Forms.ComboBox();
			this.label1 = new System.Windows.Forms.Label();
			this.stBar = new System.Windows.Forms.StatusBar();
			this.panelVersion = new System.Windows.Forms.StatusBarPanel();
			this.fswScript = new System.IO.FileSystemWatcher();
			this.fswRollback = new System.IO.FileSystemWatcher();
			this.gpMain.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.panelVersion)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.fswScript)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.fswRollback)).BeginInit();
			this.SuspendLayout();
			// 
			// txtFile
			// 
			this.txtFile.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.txtFile.Location = new System.Drawing.Point(16, 48);
			this.txtFile.Name = "txtFile";
			this.txtFile.Size = new System.Drawing.Size(368, 21);
			this.txtFile.TabIndex = 1;
			this.txtFile.Text = "";
			this.txtFile.TextChanged += new System.EventHandler(this.txtFile_TextChanged);
			// 
			// btnBrowse
			// 
			this.btnBrowse.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.btnBrowse.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnBrowse.Location = new System.Drawing.Point(392, 46);
			this.btnBrowse.Name = "btnBrowse";
			this.btnBrowse.Size = new System.Drawing.Size(32, 23);
			this.btnBrowse.TabIndex = 2;
			this.btnBrowse.Text = "...";
			this.btnBrowse.Click += new System.EventHandler(this.btnBrowse_Click);
			// 
			// lblPath
			// 
			this.lblPath.AutoSize = true;
			this.lblPath.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.lblPath.Location = new System.Drawing.Point(16, 24);
			this.lblPath.Name = "lblPath";
			this.lblPath.Size = new System.Drawing.Size(213, 17);
			this.lblPath.TabIndex = 0;
			this.lblPath.Text = "Zu installierende &Windows Installer-Datei:";
			// 
			// btnStart
			// 
			this.btnStart.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.btnStart.Enabled = false;
			this.btnStart.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnStart.Location = new System.Drawing.Point(464, 16);
			this.btnStart.Name = "btnStart";
			this.btnStart.TabIndex = 1;
			this.btnStart.Text = "&Starten...";
			this.btnStart.Click += new System.EventHandler(this.btnStart_Click);
			// 
			// btnCancel
			// 
			this.btnCancel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.btnCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.btnCancel.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnCancel.Location = new System.Drawing.Point(464, 48);
			this.btnCancel.Name = "btnCancel";
			this.btnCancel.TabIndex = 2;
			this.btnCancel.Text = "B&eenden";
			this.btnCancel.Click += new System.EventHandler(this.btnCancel_Click);
			// 
			// gpMain
			// 
			this.gpMain.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.gpMain.Controls.Add(this.rdAdvt);
			this.gpMain.Controls.Add(this.rdAdmin);
			this.gpMain.Controls.Add(this.rdDefault);
			this.gpMain.Controls.Add(this.lstState);
			this.gpMain.Controls.Add(this.txtBackupFolder);
			this.gpMain.Controls.Add(this.btnFolder);
			this.gpMain.Controls.Add(this.label3);
			this.gpMain.Controls.Add(this.label2);
			this.gpMain.Controls.Add(this.chkRollback);
			this.gpMain.Controls.Add(this.cmbDrives);
			this.gpMain.Controls.Add(this.label1);
			this.gpMain.Controls.Add(this.lblPath);
			this.gpMain.Controls.Add(this.txtFile);
			this.gpMain.Controls.Add(this.btnBrowse);
			this.gpMain.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.gpMain.Location = new System.Drawing.Point(16, 8);
			this.gpMain.Name = "gpMain";
			this.gpMain.Size = new System.Drawing.Size(440, 296);
			this.gpMain.TabIndex = 0;
			this.gpMain.TabStop = false;
			this.gpMain.Text = "Informationen zum Installationsprozess und zur berwachung";
			// 
			// rdAdvt
			// 
			this.rdAdvt.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.rdAdvt.Location = new System.Drawing.Point(24, 112);
			this.rdAdvt.Name = "rdAdvt";
			this.rdAdvt.Size = new System.Drawing.Size(136, 16);
			this.rdAdvt.TabIndex = 5;
			this.rdAdvt.Text = "Ad&vertised Installation";
			// 
			// rdAdmin
			// 
			this.rdAdmin.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.rdAdmin.Location = new System.Drawing.Point(24, 96);
			this.rdAdmin.Name = "rdAdmin";
			this.rdAdmin.Size = new System.Drawing.Size(144, 16);
			this.rdAdmin.TabIndex = 4;
			this.rdAdmin.Text = "Ad&ministrative Installation";
			// 
			// rdDefault
			// 
			this.rdDefault.Checked = true;
			this.rdDefault.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.rdDefault.Location = new System.Drawing.Point(24, 80);
			this.rdDefault.Name = "rdDefault";
			this.rdDefault.Size = new System.Drawing.Size(128, 16);
			this.rdDefault.TabIndex = 3;
			this.rdDefault.TabStop = true;
			this.rdDefault.Text = "Stan&dardinstallation";
			// 
			// lstState
			// 
			this.lstState.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.lstState.IntegralHeight = false;
			this.lstState.Location = new System.Drawing.Point(16, 210);
			this.lstState.Name = "lstState";
			this.lstState.Size = new System.Drawing.Size(408, 70);
			this.lstState.TabIndex = 13;
			this.lstState.TabStop = false;
			// 
			// txtBackupFolder
			// 
			this.txtBackupFolder.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.txtBackupFolder.BackColor = System.Drawing.SystemColors.Window;
			this.txtBackupFolder.Location = new System.Drawing.Point(16, 152);
			this.txtBackupFolder.Name = "txtBackupFolder";
			this.txtBackupFolder.ReadOnly = true;
			this.txtBackupFolder.Size = new System.Drawing.Size(368, 21);
			this.txtBackupFolder.TabIndex = 10;
			this.txtBackupFolder.TabStop = false;
			this.txtBackupFolder.Text = "";
			this.txtBackupFolder.TextChanged += new System.EventHandler(this.txtFile_TextChanged);
			// 
			// btnFolder
			// 
			this.btnFolder.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.btnFolder.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnFolder.Location = new System.Drawing.Point(392, 152);
			this.btnFolder.Name = "btnFolder";
			this.btnFolder.Size = new System.Drawing.Size(32, 23);
			this.btnFolder.TabIndex = 11;
			this.btnFolder.Text = "...";
			this.btnFolder.Click += new System.EventHandler(this.btnFolder_Click);
			// 
			// label3
			// 
			this.label3.AutoSize = true;
			this.label3.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.label3.Location = new System.Drawing.Point(16, 136);
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size(279, 17);
			this.label3.TabIndex = 9;
			this.label3.Text = "&Verzeichnis in das die Dateien gesichert werden sollen:";
			// 
			// label2
			// 
			this.label2.AutoSize = true;
			this.label2.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.label2.Location = new System.Drawing.Point(200, 104);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(54, 17);
			this.label2.TabIndex = 7;
			this.label2.Text = "&Laufwerk:";
			// 
			// chkRollback
			// 
			this.chkRollback.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.chkRollback.Location = new System.Drawing.Point(184, 80);
			this.chkRollback.Name = "chkRollback";
			this.chkRollback.Size = new System.Drawing.Size(216, 16);
			this.chkRollback.TabIndex = 6;
			this.chkRollback.Text = "&Rollbackverzeichnis auch berwachen";
			this.chkRollback.CheckedChanged += new System.EventHandler(this.chkRollback_CheckedChanged);
			// 
			// cmbDrives
			// 
			this.cmbDrives.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.cmbDrives.Enabled = false;
			this.cmbDrives.Location = new System.Drawing.Point(264, 104);
			this.cmbDrives.Name = "cmbDrives";
			this.cmbDrives.TabIndex = 8;
			// 
			// label1
			// 
			this.label1.AutoSize = true;
			this.label1.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.label1.Location = new System.Drawing.Point(16, 184);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(109, 17);
			this.label1.TabIndex = 12;
			this.label1.Text = "Stat&usinformationen:";
			// 
			// stBar
			// 
			this.stBar.Location = new System.Drawing.Point(0, 317);
			this.stBar.Name = "stBar";
			this.stBar.Panels.AddRange(new System.Windows.Forms.StatusBarPanel[] {
																					 this.panelVersion});
			this.stBar.ShowPanels = true;
			this.stBar.Size = new System.Drawing.Size(552, 22);
			this.stBar.TabIndex = 3;
			// 
			// panelVersion
			// 
			this.panelVersion.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Spring;
			this.panelVersion.BorderStyle = System.Windows.Forms.StatusBarPanelBorderStyle.None;
			this.panelVersion.Icon = ((System.Drawing.Icon)(resources.GetObject("panelVersion.Icon")));
			this.panelVersion.Width = 536;
			// 
			// fswScript
			// 
			this.fswScript.EnableRaisingEvents = true;
			this.fswScript.SynchronizingObject = this;
			this.fswScript.Deleted += new System.IO.FileSystemEventHandler(this.WatchFolder);
			this.fswScript.Changed += new System.IO.FileSystemEventHandler(this.WatchFolder);
			this.fswScript.Created += new System.IO.FileSystemEventHandler(this.WatchFolder);
			// 
			// fswRollback
			// 
			this.fswRollback.EnableRaisingEvents = true;
			this.fswRollback.SynchronizingObject = this;
			this.fswRollback.Deleted += new System.IO.FileSystemEventHandler(this.WatchFolder);
			this.fswRollback.Changed += new System.IO.FileSystemEventHandler(this.WatchFolder);
			this.fswRollback.Created += new System.IO.FileSystemEventHandler(this.WatchFolder);
			// 
			// MsiForm
			// 
			this.AcceptButton = this.btnStart;
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 14);
			this.CancelButton = this.btnCancel;
			this.ClientSize = new System.Drawing.Size(552, 339);
			this.Controls.Add(this.stBar);
			this.Controls.Add(this.gpMain);
			this.Controls.Add(this.btnCancel);
			this.Controls.Add(this.btnStart);
			this.Font = new System.Drawing.Font("Tahoma", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.MinimumSize = new System.Drawing.Size(560, 368);
			this.Name = "MsiForm";
			this.Text = "Windows Installer 3.1: Watcher";
			this.gpMain.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.panelVersion)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.fswScript)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.fswRollback)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

		#region Main
		/// <summary>
		/// Haupteinstiegspunkt der Anwendung
		/// </summary>
		[STAThread]
		static void Main(string[] args) 
		{
			string fileName = string.Empty; 

			// Argumente auswerten
			if (args.Length > 0) 
				fileName = args[0];


			// Styles aktivieren und Form anzeigen
			Application.EnableVisualStyles();
			Application.DoEvents(); 
			Application.Run(new MsiForm(fileName));
		}
		#endregion

		#region Methoden
		/// <summary>
		/// Auswhlen einer Datei wurde aktiviert
		/// </summary>
		private void btnBrowse_Click(object sender, System.EventArgs e)
		{
			using (OpenFileDialog dlgOpen = new OpenFileDialog())
			{
				// Dialog initialisieren
				dlgOpen.Title = "Windows Installer-Datei auswhlen";
				dlgOpen.Filter = "Windows Installer-Datei (*.msi)|*.msi";
				dlgOpen.FileName = this.txtFile.Text;

				// Dialog anzeigen
				if (dlgOpen.ShowDialog() == DialogResult.OK)
				{
					// Datei in Textfeld anzeigen
					this.txtFile.Text = dlgOpen.FileName; 
				}
			}
 
		}

		/// <summary>
		/// Datei oder Ordner wurde gendert
		/// </summary>
		private void txtFile_TextChanged(object sender, System.EventArgs e)
		{
			// Schaltflche aktivieren / deaktivieren
			this.btnStart.Enabled = ((this.txtFile.Text.Trim() != string.Empty) & (this.txtBackupFolder.Text.Trim() != string.Empty)) ;   
		}

		/// <summary>
		/// Abbrechen wurde aktiviert
		/// </summary>
		private void btnCancel_Click(object sender, System.EventArgs e)
		{
			// Form schlieen
			this.Close(); 
		}

		/// <summary>
		/// Installation starten und Ordner berwachen
		/// </summary>
		private void btnStart_Click(object sender, System.EventArgs e)
		{
			try
			{
				// Dateiname festlegen
				string fileName = this.txtFile.Text; 

				if (!File.Exists(fileName))
				{
					// Datei nicht vorhanden
					throw new FileNotFoundException(string.Format("Die Datei \"{0}\" konnte nicht gefunden werden",fileName), fileName); 
				}

				// Befehlszeile konstruieren
				string commandLine = string.Empty; 

				if (this.rdAdmin.Checked == true)
					commandLine = "ACTION=ADMIN";
				else if (this.rdAdvt.Checked == true)
					commandLine = "ACTION=ADVERTISE";

				// berwachung einschalten
				this.StartWatch(); 

				// Installation starten
				MsiFunctions.InstallProduct(fileName, commandLine, this.Handle, this.rdAdvt.Checked );  


			}
			catch (Exception ex)
			{
				// Exception
				MessageBox.Show(ex.Message, this.Text, MessageBoxButtons.OK, MessageBoxIcon.Information);     
			}
			finally
			{
				// berwachung ausschalten
				this.StopWatch(); 
			}
		}

		/// <summary>
		/// berwachung fr Rollback wurde gendert
		/// </summary>
		private void chkRollback_CheckedChanged(object sender, System.EventArgs e)
		{
			this.cmbDrives.Enabled = this.chkRollback.Checked ; 
		}

		/// <summary>
		/// Dialog zur Auswahl eines Ordners wurde aktiviert
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void btnFolder_Click(object sender, System.EventArgs e)
		{
			using (FolderBrowserDialog dlgFolder = new FolderBrowserDialog())
			{
				// Dialog initialisieren
				dlgFolder.Description = "Whlen Sie einen Ordner aus, in den die Skriptdateien gesichert werden sollen.";
				dlgFolder.ShowNewFolderButton = true;
				dlgFolder.SelectedPath = this.txtBackupFolder.Text;

				// Dialog anzeigen
				if (dlgFolder.ShowDialog() == DialogResult.OK)
				{
					// Datei in Textfeld anzeigen
					this.txtBackupFolder.Text = dlgFolder.SelectedPath; 
				}
			}
		}

		/// <summary>
		/// berwachung der Ordner aktivieren
		/// </summary>
		private void StartWatch()
		{
			// Liste lschen
			this.lstState.Items.Clear(); 

			// Backupordner erstellen
			if (!Directory.Exists( this.txtBackupFolder.Text ))
				Directory.CreateDirectory( this.txtBackupFolder.Text );

			// Ordner des Installers bestimmen
			string installerFolder = Path.Combine(Environment.GetEnvironmentVariable("Windir"), "Installer");

			// Eigenschaften fr berwahung setzen
			this.fswScript.Path = installerFolder;
			this.fswScript.Filter = "*.tmp";
			this.fswScript.IncludeSubdirectories = false;
			this.fswScript.EnableRaisingEvents = true;

			// Prfen ob Rollbackordner auch berwacht werden soll
			if (this.chkRollback.Checked == true)
			{
				// Rollbackordner bestimmen
				string rollbackFolder = Path.Combine(this.cmbDrives.SelectedItem.ToString(), "config.msi");

				// Falls Rollbackordner nich existiert, erstellen und Attribute setzen
				if (!Directory.Exists( rollbackFolder ))
				{
					Directory.CreateDirectory( rollbackFolder );
					File.SetAttributes(rollbackFolder, FileAttributes.Directory | FileAttributes.System | FileAttributes.Hidden);
				}

				// Eigenschaften fr berwachung (Rollback) setzen
				this.fswRollback.Path = rollbackFolder;
				this.fswRollback.Filter = "*.rbs";
				this.fswRollback.IncludeSubdirectories = false;
				this.fswRollback.EnableRaisingEvents = true;
			}
		}

		/// <summary>
		/// berwachung der Ordner beenden
		/// </summary>
		private void StopWatch()
		{
			// FSW-Komponenten deaktivieren
			this.fswRollback.EnableRaisingEvents = false;
			this.fswScript.EnableRaisingEvents = false; 
		}

		/// <summary>
		/// Event das ausgelst wird, falls eine Datei in dem Ordner gendert wird
		/// </summary>
		private void WatchFolder(object sender, System.IO.FileSystemEventArgs e)
		{
			// Informationen anzeigen
			int index = this.lstState.Items.Add(e.ChangeType.ToString() + ": " + e.FullPath);  
			this.lstState.SelectedIndex = index;

			// Auer wenn Datei gelscht wurde, kopieren
			if (e.ChangeType != WatcherChangeTypes.Deleted)
				System.IO.File.Copy(e.FullPath, System.IO.Path.Combine(this.txtBackupFolder.Text, e.Name),true);    
		
		}

		#endregion
	}
}
